@@ -24,10 +24,10 @@ module RuboCop
         include FirstElementLineBreak
 
         MSG = 'Add a line break before the first argument of a ' \
-              'multi-line method argument list.'.freeze
+              'multi-line method argument list.'
 
         def on_send(node)
-          args = node.arguments
+          args = node.arguments.dup
 
           # If there is a trailing hash arg without explicit braces, like this:
           #
@@ -36,13 +36,13 @@ module RuboCop
           # ...then each key/value pair is treated as a method 'argument'
           # when determining where line breaks should appear.
           if (last_arg = args.last)
-            if last_arg.hash_type? && !last_arg.braces?
-              args = args.concat(args.pop.children)
-            end
+            args = args.concat(args.pop.children) if last_arg.hash_type? && !last_arg.braces?
           end
 
           check_method_line_break(node, args)
         end
+        alias on_csend on_send
+        alias on_super on_send
 
         def autocorrect(node)
           EmptyLineCorrector.insert_before(node)
