@@ -12,10 +12,12 @@ module RuboCop
       #   # good
       #   !something
       class SpaceAfterNot < Cop
-        MSG = 'Do not leave space between `!` and its argument.'.freeze
+        include RangeHelp
+
+        MSG = 'Do not leave space between `!` and its argument.'
 
         def on_send(node)
-          return unless node.keyword_bang? && whitespace_after_operator?(node)
+          return unless node.prefix_bang? && whitespace_after_operator?(node)
 
           add_offense(node)
         end
