@@ -3,8 +3,8 @@
 module RuboCop
   module Cop
     module Style
-      # This cop checks for places where Integer#even? or Integer#odd?
-      # should have been used.
+      # This cop checks for places where `Integer#even?` or `Integer#odd?`
+      # can be used.
       #
       # @example
       #
@@ -15,30 +15,26 @@ module RuboCop
       #   # good
       #   if x.even?
       #   end
-      class EvenOdd < Cop
-        MSG = 'Replace with `Integer#%<method>s?`.'.freeze
+      class EvenOdd < Base
+        extend AutoCorrector
 
-        def_node_matcher :even_odd_candidate?, <<-PATTERN
+        MSG = 'Replace with `Integer#%<method>s?`.'
+
+        def_node_matcher :even_odd_candidate?, <<~PATTERN
           (send
             {(send $_ :% (int 2))
              (begin (send $_ :% (int 2)))}
             ${:== :!=}
-            (int ${0 1 2}))
+            (int ${0 1}))
         PATTERN
 
         def on_send(node)
-          even_odd_candidate?(node) do |_base_number, method, arg|
-            replacement_method = replacement_method(arg, method)
-            add_offense(node, message: format(MSG, method: replacement_method))
-          end
-        end
-
-        def autocorrect(node)
           even_odd_candidate?(node) do |base_number, method, arg|
             replacement_method = replacement_method(arg, method)
-
-            correction = "#{base_number.source}.#{replacement_method}?"
-            ->(corrector) { corrector.replace(node.source_range, correction) }
+            add_offense(node, message: format(MSG, method: replacement_method)) do |corrector|
+              correction = "#{base_number.source}.#{replacement_method}?"
+              corrector.replace(node, correction)
+            end
           end
         end
 
