@@ -14,25 +14,30 @@ module RuboCop
       #   # good
       #   a = 'a'
       #   foo if ['a', 'b', 'c'].include?(a)
-      class MultipleComparison < Cop
+      class MultipleComparison < Base
         MSG = 'Avoid comparing a variable with multiple items ' \
-          'in a conditional, use `Array#include?` instead.'.freeze
+          'in a conditional, use `Array#include?` instead.'
+
+        def on_or(node)
+          root_of_or_node = root_of_or_node(node)
+
+          return unless node == root_of_or_node
+          return unless nested_variable_comparison?(root_of_or_node)
 
-        def on_if(node)
-          return unless nested_variable_comparison?(node.condition)
           add_offense(node)
         end
 
         private
 
         def_node_matcher :simple_double_comparison?, '(send $lvar :== $lvar)'
-        def_node_matcher :simple_comparison?, <<-PATTERN
+        def_node_matcher :simple_comparison?, <<~PATTERN
           {(send $lvar :== _)
            (send _ :== $lvar)}
         PATTERN
 
         def nested_variable_comparison?(node)
           return false unless nested_comparison?(node)
+
           variables_in_node(node).count == 1
         end
 
@@ -71,6 +76,16 @@ module RuboCop
         def comparison?(node)
           simple_comparison?(node) || nested_comparison?(node)
         end
+
+        def root_of_or_node(or_node)
+          return or_node unless or_node.parent
+
+          if or_node.parent.or_type?
+            root_of_or_node(or_node.parent)
+          else
+            or_node
+          end
+        end
       end
     end
   end
