@@ -4,27 +4,27 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   subject(:cop) { described_class.new }
 
   it 'registers an offense for a nested method definition' do
-    expect_offense(<<-RUBY.strip_indent)
+    expect_offense(<<~RUBY)
       def x; def y; end; end
              ^^^^^^^^^^ Method definitions must not be nested. Use `lambda` instead.
     RUBY
   end
 
   it 'registers an offense for a nested singleton method definition' do
-    inspect_source(<<-RUBY.strip_indent)
+    expect_offense(<<~RUBY)
       class Foo
       end
       foo = Foo.new
       def foo.bar
         def baz
+        ^^^^^^^ Method definitions must not be nested. Use `lambda` instead.
         end
       end
     RUBY
-    expect(cop.offenses.size).to eq(1)
   end
 
   it 'registers an offense for a nested method definition inside lambda' do
-    expect_offense(<<-RUBY.strip_indent)
+    expect_offense(<<~RUBY)
       def foo
         bar = -> { def baz; puts; end }
                    ^^^^^^^^^^^^^^^^^^ Method definitions must not be nested. Use `lambda` instead.
@@ -33,19 +33,19 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   end
 
   it 'registers an offense for a nested class method definition' do
-    inspect_source(<<-RUBY.strip_indent)
+    expect_offense(<<~RUBY)
       class Foo
         def self.x
           def self.y
+          ^^^^^^^^^^ Method definitions must not be nested. Use `lambda` instead.
           end
         end
       end
     RUBY
-    expect(cop.offenses.size).to eq(1)
   end
 
   it 'does not register an offense for a lambda definition inside method' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       def foo
         bar = -> { puts  }
         bar.call
@@ -54,7 +54,7 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   end
 
   it 'does not register offense for nested definition inside instance_eval' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def x(obj)
           obj.instance_eval do
@@ -67,7 +67,7 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   end
 
   it 'does not register offense for nested definition inside instance_exec' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def x(obj)
           obj.instance_exec do
@@ -80,7 +80,7 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   end
 
   it 'does not register offense for definition of method on local var' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def x(obj)
           def obj.y
@@ -91,7 +91,7 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   end
 
   it 'does not register offense for nested definition inside class_eval' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def x(klass)
           klass.class_eval do
@@ -104,7 +104,7 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   end
 
   it 'does not register offense for nested definition inside class_exec' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def x(klass)
           klass.class_exec do
@@ -117,7 +117,7 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   end
 
   it 'does not register offense for nested definition inside module_eval' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def self.define(mod)
           mod.module_eval do
@@ -130,7 +130,7 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   end
 
   it 'does not register offense for nested definition inside module_exec' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def self.define(mod)
           mod.module_exec do
@@ -143,7 +143,7 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   end
 
   it 'does not register offense for nested definition inside class shovel' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def bar
           class << self
@@ -156,7 +156,7 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
   end
 
   it 'does not register offense for nested definition inside Class.new' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def self.define
           Class.new(S) do
@@ -177,8 +177,30 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
     RUBY
   end
 
+  it 'does not register offense for nested definition inside ::Class.new' do
+    expect_no_offenses(<<~RUBY)
+      class Foo
+        def self.define
+          ::Class.new(S) do
+            def y
+            end
+          end
+        end
+      end
+
+      class Foo
+        def self.define
+          ::Class.new do
+            def y
+            end
+          end
+        end
+      end
+    RUBY
+  end
+
   it 'does not register offense for nested definition inside Module.new' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def self.define
           Module.new do
@@ -190,8 +212,21 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
     RUBY
   end
 
+  it 'does not register offense for nested definition inside ::Module.new' do
+    expect_no_offenses(<<~RUBY)
+      class Foo
+        def self.define
+          ::Module.new do
+            def y
+            end
+          end
+        end
+      end
+    RUBY
+  end
+
   it 'does not register offense for nested definition inside Struct.new' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Foo
         def self.define
           Struct.new(:name) do
@@ -211,4 +246,26 @@ RSpec.describe RuboCop::Cop::Lint::NestedMethodDefinition do
       end
     RUBY
   end
+
+  it 'does not register offense for nested definition inside ::Struct.new' do
+    expect_no_offenses(<<~RUBY)
+      class Foo
+        def self.define
+          ::Struct.new(:name) do
+            def y
+            end
+          end
+        end
+      end
+
+      class Foo
+        def self.define
+          ::Struct.new do
+            def y
+            end
+          end
+        end
+      end
+    RUBY
+  end
 end
