@@ -1,13 +1,12 @@
 # frozen_string_literal: true
 
 RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
-  subject(:cop) { described_class.new(config) }
-
   let(:cop_config) { { 'Max' => 5, 'CountComments' => false } }
 
   it 'rejects a class with more than 5 lines' do
-    inspect_source(<<-RUBY.strip_indent)
+    expect_offense(<<~RUBY)
       class Test
+      ^^^^^^^^^^ Class has too many lines. [6/5]
         a = 1
         a = 2
         a = 3
@@ -16,13 +15,10 @@ RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
         a = 6
       end
     RUBY
-    expect(cop.offenses.size).to eq(1)
-    expect(cop.messages).to eq(['Class has too many lines. [6/5]'])
-    expect(cop.config_to_allow_offenses).to eq('Max' => 6)
   end
 
   it 'reports the correct beginning and end lines' do
-    inspect_source(<<-RUBY.strip_indent)
+    offenses = inspect_source(<<~RUBY)
       class Test
         a = 1
         a = 2
@@ -33,13 +29,13 @@ RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
       end
     RUBY
 
-    offense = cop.offenses.first
+    offense = offenses.first
     expect(offense.location.first_line).to eq(1)
     expect(offense.location.last_line).to eq(8)
   end
 
   it 'accepts a class with 5 lines' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Test
         a = 1
         a = 2
@@ -51,7 +47,7 @@ RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
   end
 
   it 'accepts a class with less than 5 lines' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Test
         a = 1
         a = 2
@@ -62,7 +58,7 @@ RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
   end
 
   it 'does not count blank lines' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Test
         a = 1
         a = 2
@@ -76,7 +72,7 @@ RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
   end
 
   it 'accepts empty classes' do
-    expect_no_offenses(<<-RUBY.strip_indent)
+    expect_no_offenses(<<~RUBY)
       class Test
       end
     RUBY
@@ -84,7 +80,7 @@ RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
 
   context 'when a class has inner classes' do
     it 'does not count lines of inner classes' do
-      expect_no_offenses(<<-RUBY.strip_indent)
+      expect_no_offenses(<<~RUBY)
         class NamespaceClass
           class TestOne
             a = 1
@@ -110,8 +106,9 @@ RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
     end
 
     it 'rejects a class with 6 lines that belong to the class directly' do
-      inspect_source(<<-RUBY.strip_indent)
+      expect_offense(<<~RUBY)
         class NamespaceClass
+        ^^^^^^^^^^^^^^^^^^^^ Class has too many lines. [6/5]
           class TestOne
             a = 1
             a = 2
@@ -134,7 +131,6 @@ RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
           a = 6
         end
       RUBY
-      expect(cop.offenses.size).to eq(1)
     end
   end
 
@@ -142,8 +138,9 @@ RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
     before { cop_config['CountComments'] = true }
 
     it 'also counts commented lines' do
-      inspect_source(<<-RUBY.strip_indent)
+      expect_offense(<<~RUBY)
         class Test
+        ^^^^^^^^^^ Class has too many lines. [6/5]
           a = 1
           #a = 2
           a = 3
@@ -152,7 +149,56 @@ RSpec.describe RuboCop::Cop::Metrics::ClassLength, :config do
           a = 6
         end
       RUBY
-      expect(cop.offenses.size).to eq(1)
+    end
+  end
+
+  context 'when `CountAsOne` is not empty' do
+    before { cop_config['CountAsOne'] = ['array'] }
+
+    it 'folds array into one line' do
+      expect_no_offenses(<<~RUBY)
+        class Test
+          a = 1
+          a = [
+            2,
+            3,
+            4,
+            5
+          ]
+        end
+      RUBY
+    end
+  end
+
+  context 'when inspecting a class defined with Class.new' do
+    it 'registers an offense' do
+      expect_offense(<<~RUBY)
+        Foo = Class.new do
+        ^^^ Class has too many lines. [6/5]
+          a = 1
+          a = 2
+          a = 3
+          a = 4
+          a = 5
+          a = 6
+        end
+      RUBY
+    end
+  end
+
+  context 'when inspecting a class defined with ::Class.new' do
+    it 'registers an offense' do
+      expect_offense(<<~RUBY)
+        Foo = ::Class.new do
+        ^^^ Class has too many lines. [6/5]
+          a = 1
+          a = 2
+          a = 3
+          a = 4
+          a = 5
+          a = 6
+        end
+      RUBY
     end
   end
 end
